///////////////////////////////////////////////////////////////////////
// SleepIntercept.cpp
//
// Applied Data Systems
// ------------------------------------------------------------------
// 720020-1167x
//
// See SleepIntercept.h for a description and history
///////////////////////////////////////////////////////////////////////
#include "SleepIntercept.h"

SleepIntercept::SleepIntercept()
{
	m_stop = TRUE;
	m_threadID = 0;
}

BOOL SleepIntercept::Enable()
{
	// get the current 'VddPwrOffFromApps' setting to make sure the current value is
	// set to support this.  
	// 
	// We cannot modify the key at run time because the Power driver reads the key 
	// only at startup.  
	// We cannot call DeactivateDevice, change the key and then call ActivateDevice 
	// to force the driver to reload the key value because we don't have a handle 
	// to the device handle and the handle in the /Drivers/Active reg key is only 
	// walid for the driver's memory space.
	if(!GetPowerRegVal())
	{
		RETAILMSG(TRUE, (_T("\r\n ** Cannot intercept the RQONOFF interrupt! **\r\n")));
		RETAILMSG(TRUE, (_T(" You must set the following registry key to enable the feature:\r\n")));
		RETAILMSG(TRUE, (_T("\t[HKEY_LOCAL_MACHINE\\%s]\r\n"), REG_POWER_KEY));
		RETAILMSG(TRUE, (_T("\t\t%s=dword:1\r\n"), REG_POWER_VAL));
		return FALSE;
	}

	// only allow enabling once
	if(m_threadID == 0)
	{
		// set the reg key to get the event and stop system handling of RQONOFF

		m_stop = FALSE;
		CreateThread(NULL, 0, _InterceptProc, this, 0, &m_threadID);
	}

	return TRUE;
}

void SleepIntercept::Disable()
{
	// only disable if a thread is running
	if(m_threadID != 0)
	{
		// set the reg key to use system handling of RQONOFF

		// signal the thread to stop
		m_stop = TRUE;
		m_threadID = 0;
	}
}

void SleepIntercept::SetWorkFunction(WORK_FUNCTION fxn)
{
	// save the callback function
	m_fxn = fxn;
}

DWORD SleepIntercept::InterceptProc()
{
	HANDLE hPowerOffEvent = CreateEvent(NULL, TRUE, FALSE, POWER_OFF_EVENT_NAME);

	while(!m_stop)
	{
		// wait for only 1 sec to allow thread termination
		if(WaitForSingleObject(hPowerOffEvent, 1000) == WAIT_OBJECT_0)
		{
			// call the callback functiuon if it's not null
			if(m_fxn != NULL)
			{
				m_fxn();
			}

			// go to sleep
			keybd_event(VK_OFF, 0, 0, 0);

			// when we wake, we'll come back here and begin waiting again
		}
	}

	// clean up
	CloseHandle(hPowerOffEvent);

	// exit the thread
	ExitThread(0);

	return 0;
}

DWORD SleepIntercept::_InterceptProc(void *thisPtr)
{
	return ((SleepIntercept*)thisPtr)->InterceptProc();
}

BOOL SleepIntercept::GetPowerRegVal()
{
	HKEY key;
	DWORD value;
	DWORD size = sizeof(DWORD);

	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, REG_POWER_KEY, 0, 0, &key) != ERROR_SUCCESS)
		return FALSE;


	if(RegQueryValueEx(key, REG_POWER_VAL, 0, NULL, (BYTE*)&value, &size) != ERROR_SUCCESS)
		return FALSE;

	return (value != 0);
}