///////////////////////////////////////////////////////////////////////
// SleepIntercept.h
//
// Applied Data Systems
// ------------------------------------------------------------------
// 720020-1167x
//
// Description
// -----------
// SleepIntercept is a helper class that allows your application to do work
// after an RQONOFF (suspend) interrupt occurs and before the system actually
// goes to sleep.
//
// To use the class you *must* be using a CE 4.x ADS device and you must set
// the following registry key:
//    [HKEY_LOCAL_MACHINE]\Drivers\BuiltIn\Power
//          VddPwrOffFromApps=dword:1
//
// This code can be modified to work with CE 3.0 by changing the reg keys.  For
// more information see the On_Off.exe sample app (720020-1103x)
//
// History
// -------
// 720020-11671 - Dec 22, 2003 : ctacke
//   - Initial implementation
//
///////////////////////////////////////////////////////////////////////

#include <windows.h>

// work function pointer typedef
typedef void (*WORK_FUNCTION)(void);

// enabling keys - defined by ADS - do not change
#define REG_POWER_KEY _T("Drivers\\BuiltIn\\Power")
#define REG_POWER_VAL _T("VddPwrOffFromApps")

// system event name - defined by ADS - do not change
#define POWER_OFF_EVENT_NAME _T("VDDPWROFF")

class SleepIntercept
{
private:
	WORK_FUNCTION m_fxn;
	DWORD InterceptProc();
	DWORD m_threadID;
	BOOL GetPowerRegVal();
	volatile BOOL m_stop;
	static DWORD _InterceptProc(void*);

public:
	SleepIntercept();
	BOOL Enable();
	void Disable();
	void SetWorkFunction(WORK_FUNCTION fxn);
};
